@@ -28,15 +28,12 @@
 
 #if ENABLE(B3_JIT)
 
-#include "B3BasicBlockInlines.h"
+#include "B3BasicBlock.h"
 #include "B3BreakCriticalEdges.h"
-#include "B3Dominators.h"
 #include "B3FixSSA.h"
-#include "B3InsertionSetInlines.h"
+#include "B3InsertionSet.h"
 #include "B3PhaseScope.h"
-#include "B3ProcedureInlines.h"
-#include "B3SwitchValue.h"
-#include "B3UpsilonValue.h"
+#include "B3Procedure.h"
 #include "B3ValueInlines.h"
 #include <wtf/IndexSet.h>
 
@@ -80,7 +77,16 @@ public:
             if (block->last()->type() != Void) // Demoting doesn't handle terminals with values.
                 continue;
 
-            candidates.add(block);
+            bool canCopyBlock = true;
+            for (Value* value : *block) {
+                if (value->kind().isCloningForbidden()) {
+                    canCopyBlock = false;
+                    break;
+                }
+            }
+
+            if (canCopyBlock)
+                candidates.add(block);
         }
 
         // Collect the set of values that must be de-SSA'd.
