@@ -23,11 +23,12 @@
 #include "config.h"
 #include "MouseRelatedEvent.h"
 
-#include "DOMWindow.h"
 #include "Document.h"
-#include "Frame.h"
-#include "FrameView.h"
+#include "EventNames.h"
 #include "LayoutPoint.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "RenderLayer.h"
 #include "RenderObject.h"
 #include <wtf/IsoMallocInlines.h>
@@ -38,10 +39,11 @@ WTF_MAKE_ISO_ALLOCATED_IMPL(MouseRelatedEvent);
 
 MouseRelatedEvent::MouseRelatedEvent(const AtomString& eventType, CanBubble canBubble, IsCancelable isCancelable, IsComposed isComposed,
     MonotonicTime timestamp, RefPtr<WindowProxy>&& view, int detail,
-    const IntPoint& screenLocation, const IntPoint& windowLocation, const IntPoint& movementDelta, OptionSet<Modifier> modifiers, IsSimulated isSimulated, IsTrusted isTrusted)
+    const IntPoint& screenLocation, const IntPoint& windowLocation, double movementX, double movementY, OptionSet<Modifier> modifiers, IsSimulated isSimulated, IsTrusted isTrusted)
     : UIEventWithKeyState(eventType, canBubble, isCancelable, isComposed, timestamp, WTFMove(view), detail, modifiers, isTrusted)
     , m_screenLocation(screenLocation)
-    , m_movementDelta(movementDelta)
+    , m_movementX(movementX)
+    , m_movementY(movementY)
     , m_isSimulated(isSimulated == IsSimulated::Yes)
 {
     init(m_isSimulated, windowLocation);
@@ -49,19 +51,28 @@ MouseRelatedEvent::MouseRelatedEvent(const AtomString& eventType, CanBubble canB
 
 MouseRelatedEvent::MouseRelatedEvent(const AtomString& type, IsCancelable isCancelable, MonotonicTime timestamp, RefPtr<WindowProxy>&& view, const IntPoint& globalLocation, OptionSet<Modifier> modifiers)
     : MouseRelatedEvent(type, CanBubble::Yes, isCancelable, IsComposed::Yes, timestamp,
-        WTFMove(view), 0, globalLocation, globalLocation /* Converted in init */, { }, modifiers, IsSimulated::No)
+        WTFMove(view), 0, globalLocation, globalLocation /* Converted in init */, 0, 0, modifiers, IsSimulated::No)
 {
 }
 
 MouseRelatedEvent::MouseRelatedEvent(const AtomString& eventType, const MouseRelatedEventInit& initializer, IsTrusted isTrusted)
     : UIEventWithKeyState(eventType, initializer)
     , m_screenLocation(IntPoint(initializer.screenX, initializer.screenY))
-    , m_movementDelta(IntPoint(0, 0))
+    , m_movementX(initializer.movementX)
+    , m_movementY(initializer.movementY)
 {
     ASSERT_UNUSED(isTrusted, isTrusted == IsTrusted::No);
     init(false, IntPoint(0, 0));
 }
 
+static inline bool isMoveEventType(const AtomString& eventType)
+{
+    auto& eventNames = WebCore::eventNames();
+    return eventType == eventNames.mousemoveEvent
+        || eventType == eventNames.pointermoveEvent
+        || eventType == eventNames.touchmoveEvent;
+}
+
 void MouseRelatedEvent::init(bool isSimulated, const IntPoint& windowLocation)
 {
     if (!isSimulated) {
@@ -74,6 +85,11 @@ void MouseRelatedEvent::init(bool isSimulated, const IntPoint& windowLocation)
     }
 
     initCoordinates();
+
+    if (!isConstructedFromInitializer() && !isMoveEventType(type())) {
+        m_movementX = 0;
+        m_movementY = 0;
+    }
 }
 
 void MouseRelatedEvent::initCoordinates()
@@ -87,16 +103,16 @@ void MouseRelatedEvent::initCoordinates()
     m_hasCachedRelativePosition = false;
 }
 
-FrameView* MouseRelatedEvent::frameViewFromWindowProxy(WindowProxy* windowProxy)
+LocalFrameView* MouseRelatedEvent::frameViewFromWindowProxy(WindowProxy* windowProxy)
 {
-    if (!windowProxy || !is<DOMWindow>(windowProxy->window()))
+    if (!windowProxy || !is<LocalDOMWindow>(windowProxy->window()))
         return nullptr;
 
-    auto* frame = downcast<DOMWindow>(*windowProxy->window()).frame();
+    auto* frame = downcast<LocalDOMWindow>(*windowProxy->window()).frame();
     return frame ? frame->view() : nullptr;
 }
 
-LayoutPoint MouseRelatedEvent::pagePointToClientPoint(LayoutPoint pagePoint, FrameView* frameView)
+LayoutPoint MouseRelatedEvent::pagePointToClientPoint(LayoutPoint pagePoint, LocalFrameView* frameView)
 {
     if (!frameView)
         return pagePoint;
@@ -104,7 +120,7 @@ LayoutPoint MouseRelatedEvent::pagePointToClientPoint(LayoutPoint pagePoint, Fra
     return flooredLayoutPoint(frameView->documentToClientPoint(pagePoint));
 }
 
-LayoutPoint MouseRelatedEvent::pagePointToAbsolutePoint(LayoutPoint pagePoint, FrameView* frameView)
+LayoutPoint MouseRelatedEvent::pagePointToAbsolutePoint(LayoutPoint pagePoint, LocalFrameView* frameView)
 {
     if (!frameView)
         return pagePoint;
